window.app = {
    server: "http://painel.taxisimples.com.br",
    timeStatusRequest: 5e3,
    company_id: 912922731
};

window.device = false;

store.set("currentMarker", undefined);

var txsimples = angular.module("txsimples", [ "ngResource", "ionic", "animate" ]);

document.addEventListener("deviceready", function() {
    window.device = true;
    if (navigator.network.connection.type == Connection.NONE) {
        alert("Não foi possivel estabelecer conexão com o servidor.");
    } else {
        console.log(navigator.network.connection.type);
    }
});

txsimples.run([ "$rootScope", "$state", "Protected", function($rootScope, $state, Protected) {
    $rootScope.goTo = function(page) {
        console.log("Going to " + page);
        $state.go(page);
    };
    $rootScope.cleanUp = function() {
        store.set("source", undefined);
        store.set("destination", undefined);
        store.set("estimate", undefined);
    };
    $rootScope.registered = function() {
        if (Protected.auth() === false) {}
        return Protected.auth();
    };
} ]);

txsimples.config(function($compileProvider) {
    $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel):/);
});

txsimples.config(function($stateProvider, $urlRouterProvider) {
    $stateProvider.state("intro", {
        url: "/intro",
        templateUrl: "templates/intro.html",
        controller: "IntroCtrl"
    }).state("pedir", {
        url: "/pedir",
        templateUrl: "templates/require.html",
        controller: "RequireCtrl"
    }).state("registro", {
        url: "/registro",
        templateUrl: "templates/register.html",
        controller: "UserCtrl"
    }).state("definicoes", {
        url: "/definicoes",
        templateUrl: "templates/defines.html",
        controller: "UserCtrl"
    }).state("confirmar", {
        url: "/confirmar",
        templateUrl: "templates/confirm.html",
        controller: "ConfirmCtrl"
    }).state("acompanhar", {
        url: "/acompanhar",
        templateUrl: "templates/follow.html",
        controller: "FollowCtrl"
    });
    $urlRouterProvider.otherwise("/intro");
});

txsimples.service("Facebook", [ "$q", "device", function($q, device) {
    return {
        login: function(permissions) {
            permissions = permissions || [ "email", "public_profile" ];
            var defer = $q.defer();
            device.then(function() {
                facebookConnectPlugin.login(permissions, function(data) {
                    defer.resolve(data);
                }, function(err) {
                    defer.reject(err);
                });
            });
            return defer.promise;
        },
        api: function(query, permissions) {
            permissions = permissions || [ "email", "public_profile" ];
            var defer = $q.defer();
            device.then(function() {
                facebookConnectPlugin.api(query, permissions, function(data) {
                    defer.resolve(data);
                }, function(err) {
                    defer.reject(err);
                });
            });
            return defer.promise;
        },
        getAccessToken: function() {
            var defer = $q.defer();
            device.then(function() {
                facebookConnectPlugin.getAccessToken(function(data) {
                    defer.resolve(data);
                }, function(err) {
                    defer.reject(err);
                });
            });
            return defer.promise;
        },
        showDialog: function(options) {
            var defer = $q.defer();
            device.then(function() {
                facebookConnectPlugin.showDialog(options, function(data) {
                    defer.resolve(data);
                }, function(err) {
                    defer.reject(err);
                });
            });
            return defer.promise;
        }
    };
} ]);

txsimples.service("Loading", function($ionicLoading) {
    this.show = function() {
        this.loading = $ionicLoading.show({
            content: '<img src="img/loading.gif" alt="Loading" />',
            animation: "fade-in",
            showBackdrop: true,
            maxWidth: 200,
            showDelay: 500
        });
    };
    this.hide = function() {
        this.loading.hide();
    };
});

txsimples.service("Map", function($q, $timeout, Ride) {
    var loc = [ 0, 0 ], markers = [], geocoder = undefined, scope = this;
    this.initialize = function($q) {
        geocoder = new google.maps.Geocoder();
        var mapOptions = {
            center: new google.maps.LatLng(-3.7630658, -38.5211885),
            zoom: 13,
            disableDefaultUI: true,
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        scope.map = [];
        _.filter(document.getElementsByClassName("map"), function(elem) {
            scope.map.push(new google.maps.Map(elem, mapOptions));
        });
    };
    this.update = function() {
        _.each(scope.map, function(map) {
            google.maps.event.trigger(map, "resize");
        });
    };
    this.setMarkers = function(map) {
        for (var i = 0; i < markers.length; i++) {
            markers[i].setMap(map);
        }
    };
    this.clearMarkers = function() {
        scope.setMarkers(null);
    };
    this.addMarker = function(loc, clear) {
        if (clear) {
            scope.clearMarkers();
        }
        var marker = {
            position: new google.maps.LatLng(loc[0], loc[1]),
            draggable: false
        };
        if (loc[2]) {
            marker.icon = loc[2];
        }
        _.each(scope.map, function(map) {
            marker.map = map;
            markers.push(new google.maps.Marker(marker));
        });
        store.set("currentMarker", loc);
    };
    this.getCurrentPosition = function() {
        var position = $q.defer();
        navigator.geolocation.getCurrentPosition(function(pos) {
            position.resolve(pos);
        }, function(err) {
            position.reject(err);
        }, {
            enableHighAccuracy: true
        });
        $timeout(function() {
            position.reject({
                timeout: true
            });
        }, 5e3);
        return position.promise;
    };
    this.centerOn = function(lat, lng) {
        if (lat && lng) {
            var loc = new google.maps.LatLng(lat, lng);
            _.each(scope.map, function(map) {
                map.setCenter(loc);
                map.panTo(loc);
            });
        } else {
            scope.getCurrentPosition(function(lat, lng) {
                var loc = new google.maps.LatLng(lat, lng);
                _.each(scope.map, function(map) {
                    map.setCenter(loc);
                    map.panTo(loc);
                });
            });
        }
    };
    this.currentPosition = function() {
        var position = scope.getCurrentPosition(), deferred = $q.defer();
        position.then(function(pos) {
            var pos = {
                lat: pos.coords.latitude,
                lng: pos.coords.longitude
            };
            scope.addMarker([ pos.lat, pos.lng ], true);
            scope.centerOn(pos.lat, pos.lng);
            deferred.resolve(pos);
        }, function() {
            deferred.reject();
        });
        scope.update();
        return deferred.promise;
    };
    this.geocode = function(pos) {
        var position = new google.maps.LatLng(pos.lat, pos.lng), address = $q.defer();
        geocoder.geocode({
            latLng: position
        }, function(results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                var resolved = false;
                angular.forEach(results, function(result) {
                    if (!resolved) {
                        var locate = Ride.placeObject(result);
                        if (locate.city && locate.street && locate.neighborhood) {
                            address.resolve(result);
                            resolved = true;
                        }
                    }
                });
                if (!resolved) {
                    address.reject();
                }
            } else {
                alert("Geocoder failed due to: " + status);
            }
        });
        return address.promise;
    };
    this.clone = function() {
        var loc = store.get("currentMarker");
        if (loc !== undefined) {
            this.addMarker(loc, true);
            this.centerOn(loc[0], loc[1]);
        }
    };
});

txsimples.service("Protected", function($q) {
    this.user = {
        name: "",
        email: "",
        phone: "",
        company_id: 0,
        registered: false
    };
    this.getUser = function() {
        if (store.get("user") === undefined) {
            return this.setUser(this.user);
        } else {
            return store.get("user");
        }
    };
    this.setUser = function(user) {
        this.user = user;
        return store.set("user", user);
    };
    this.auth = function() {
        return this.getUser().registered;
    };
});

"use strict";

txsimples.service("Ride", function($q, $resource, RideResource, Facebook) {
    var self = this;
    var directionService = new google.maps.DirectionsService();
    this.route = function(source, destination) {
        var route = $q.defer();
        directionService.route({
            origin: new google.maps.LatLng(source[0], source[1]),
            destination: new google.maps.LatLng(destination[0], destination[1]),
            travelMode: google.maps.DirectionsTravelMode.DRIVING
        }, function(res, status) {
            if (status == google.maps.DirectionsStatus.OK) {
                var r = {};
                r.distance = res.routes[0].legs[0].distance.value;
                r.distance = r.distance.toFixed(2);
                r.directions = res;
                route.resolve(r);
            } else {
                route.reject();
            }
        });
        return route.promise;
    };
    this.price = function(city, distance) {
        var price = $q.defer();
        RideResource.estimatePrice({
            distance: distance,
            city: city
        }, function(r) {
            price.resolve(r);
        }, function(err) {
            price.reject(err);
        });
        return price.promise;
    };
    this.placeObject = function(point) {
        var address = point.address_components, location = _.toArray(point.geometry.location), result = {};
        result.latitude = location[0];
        result.longitude = location[1];
        for (var i = 0; i < address.length; i++) {
            if (address[i].types.indexOf("route") != -1) {
                result.street = address[i].long_name;
            } else if (address[i].types.indexOf("street_number") != -1) {
                result.number = address[i].long_name;
            } else if (address[i].types.indexOf("sublocality") != -1) {
                result.neighborhood = address[i].long_name;
            } else if (address[i].types.indexOf("neighborhood") != -1) {
                result.neighborhood = address[i].long_name;
            } else if (address[i].types.indexOf("locality") != -1) {
                result.city = address[i].long_name;
            } else if (address[i].types.indexOf("administrative_area_level_1") != -1) {
                result.state = address[i].long_name;
            }
        }
        return result;
    };
    this.estimate = function() {
        var estimate = $q.defer();
        var source = self.placeObject(store.get("source"));
        var route = self.route(_.toArray(store.get("source").geometry.location), _.toArray(store.get("destination").geometry.location));
        route.then(function(route) {
            var price = self.price(source.city, route.distance);
            price.then(function(r) {
                store.set("estimate", r.response);
                estimate.resolve(r);
            }, function(err) {
                estimate.reject(err);
            });
        });
        return estimate.promise;
    };
    this.submitPlaces = function(arrs) {
        var points = [];
        angular.forEach(arrs, function(place) {
            var deferred = $q.defer();
            Facebook.getAccessToken().then(function(token) {
                RideResource.createPoint({
                    access_token: token,
                    point: self.placeObject(place)
                }, function(r) {
                    deferred.resolve(r);
                }, function(err) {
                    deferred.reject(err);
                });
            });
            points.push(deferred.promise);
        });
        return $q.all(points);
    };
    this.requireCab = function(points) {
        var result = $q.defer(), destination = undefined;
        if (points.length > 1) {
            destination = points[1].id;
        }
        var ride = $q.defer(), rideData = {
            source_id: points[0].id,
            destination_id: destination,
            description: "[description]"
        };
        Facebook.getAccessToken().then(function(token) {
            rideData.access_token = token;
            RideResource.callRide(rideData, function(res) {
                store.set("ride", res);
                store.set("waiting", true);
                result.resolve(res);
            }, function(err) {
                result.reject(err);
            });
        });
        return result.promise;
    };
    this.cancel = function(ride) {
        var deferred = $q.defer();
        Facebook.getAccessToken().then(function(token) {
            RideResource.cancel({
                id: ride,
                access_token: token
            }, function(r) {
                store.set("waiting", false);
                deferred.resolve(r);
            }, function(err) {
                deferred.reject(err);
            });
        });
        return deferred.promise;
    };
});

txsimples.service("Translate", [ "$q", "device", function($q, device) {
    var messages = {
        "has already been taken": "já está em uso"
    };
    return {
        msg: function(msg) {
            return messages[msg] || msg;
        }
    };
} ]);

txsimples.factory("device", function($q) {
    var defer = $q.defer();
    document.addEventListener("deviceready", function() {
        defer.resolve();
    }, false);
    return defer.promise;
});

txsimples.factory("RideResource", function($resource, Facebook) {
    var options = {
        company_id: app.company_id
    };
    var methods = {
        callRide: {
            url: app.server + "/clients/dispatch_ride",
            method: "POST"
        },
        createPoint: {
            url: app.server + "/clients/create_point",
            method: "POST"
        },
        checkRide: {
            url: app.server + "/cab_request/status",
            method: "GET"
        },
        cancel: {
            url: app.server + "/clients/cancel_ride",
            method: "POST"
        },
        getCompanies: {
            url: app.server + "/companies.json",
            method: "GET",
            isArray: true
        },
        estimatePrice: {
            url: app.server + "/price/city/:city/:distance",
            method: "GET"
        }
    };
    return $resource(app.server + "/clients/:id", options, methods);
});

txsimples.factory("UserService", function($resource, Facebook) {
    var options = {
        company_id: app.company_id
    };
    var methods = {
        join: {
            url: app.server + "/clients/join",
            method: "POST"
        },
        me: {
            url: app.server + "/clients/me",
            method: "GET"
        },
        edit: {
            url: app.server + "/clients/:id.json",
            method: "PUT"
        }
    };
    return $resource(app.server + "/clients/:id", options, methods);
});

txsimples.controller("AboutCtrl", [ "$scope", function($scope) {} ]);

var source = new google.maps.Geocoder();

txsimples.controller("AddressCtrl", [ "$scope", "$q", "$timeout", "$ionicModal", "$ionicPopup", "Map", "Loading", "Protected", function($scope, $q, $timeout, $ionicModal, $ionicPopup, Map, Loading, Protected) {
    $scope.address = undefined;
    $scope.addresses = [];
    $scope.user = Protected.getUser();
    $scope.points = $scope.user.favorites;
    $ionicModal.fromTemplateUrl("modals.address", function(modal) {
        $scope.modal = modal;
    }, {
        scope: $scope,
        animation: "active"
    });
    $scope.selectPoint = function() {
        var place = {
            address_components: [ {
                long_name: $scope.currentPoint.number,
                short_name: $scope.currentPoint.number,
                types: [ "street_number" ]
            }, {
                long_name: $scope.currentPoint.street,
                short_name: $scope.currentPoint.street,
                types: [ "route" ]
            }, {
                long_name: $scope.currentPoint.neighborhood,
                short_name: $scope.currentPoint.neighborhood,
                types: [ "neighborhood", "political" ]
            }, {
                long_name: $scope.currentPoint.city,
                short_name: $scope.currentPoint.city,
                types: [ "locality", "political" ]
            }, {
                long_name: $scope.currentPoint.city,
                short_name: $scope.currentPoint.city,
                types: [ "administrative_area_level_2", "political" ]
            }, {
                long_name: $scope.currentPoint.state,
                short_name: $scope.currentPoint.state,
                types: [ "administrative_area_level_1", "political" ]
            }, {
                long_name: "Brasil",
                short_name: "BR",
                types: [ "country", "political" ]
            } ],
            formatted_address: $scope.currentPoint.street + ", " + $scope.currentPoint.number + " - " + $scope.currentPoint.neighborhood + ", " + $scope.currentPoint.city + "- " + $scope.currentPoint.state + ", Brasil",
            geometry: {
                location: {
                    k: $scope.currentPoint.latitude,
                    D: $scope.currentPoint.longitude
                },
                location_type: "RANGE_INTERPOLATED"
            },
            types: [ "street_address" ],
            street_number: $scope.currentPoint.number
        };
        $scope.confirm(place, true);
        console.log($scope.currentPoint);
        $scope.currentPoint = null;
    };
    $scope.openModal = function() {
        console.log($scope.modal.el);
        $scope.modal.show();
    };
    $scope.closeModal = function() {
        console.log($scope.modal.el);
        $scope.modal.hide();
    };
    $scope.currentPosition = function() {
        Loading.show();
        Map.currentPosition().then(function(position) {
            Map.geocode(position).then(function(place) {
                $scope.confirm(place);
                Loading.hide();
            }, function() {
                Loading.hide();
                $ionicPopup.alert({
                    title: "Endereço não encontrado.",
                    content: "Não pudemos encontrar um endereço para sua localição."
                });
            });
        }, function() {
            Loading.hide();
            $ionicPopup.show({
                title: "Serviço de Localização desativado.",
                content: "Essa função requer que você use o serviço de localização do seu dispositivo.",
                buttons: [ {
                    text: "OK",
                    type: "button-peter-rider",
                    onTap: function(e) {
                        return true;
                    }
                } ]
            });
        });
    };
    $scope.findPlace = function(address) {
        var r = $q.defer();
        source.geocode({
            address: address + ", Fortaleza, Ceara, Brasil",
            language: "PT-BR"
        }, function(results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                $scope.addresses = [];
                _.each(results, function(result) {
                    $scope.addresses.push(result);
                });
                r.resolve(results);
            }
        });
        return r.promise;
    };
    $scope.renderInMap = function(place) {
        var location = _.toArray(place.geometry.location);
        Map.addMarker(location, true);
        Map.centerOn(location[0], location[1]);
    };
    $scope.confirm = function(place, renderIt) {
        var arr = place.formatted_address.split("-", 1)[0].split(",", 2), pos = store.get("togglePlace"), address = arr[0], num = parseInt(arr[1]);
        $scope.closeModal();
        store.set(pos, place);
        if (renderIt) {
            $scope.renderInMap(place);
        }
        if (!_.isNaN(num)) {
            store.set(pos, _.extend(store.get(pos), {
                street_number: num
            }));
            $timeout(function() {
                $(".enter-" + pos + " .num").val(num);
            }, 500);
        }
        $timeout(function() {
            $(".enter-" + pos + " .address").val(address);
        }, 500);
    };
    $scope.$watch("num", function(num) {
        if (num) {
            $scope.update(num);
        }
    });
    $scope.update = function(num) {
        var pos = store.get("togglePlace"), address = store.get(pos).formatted_address.split("-", 1)[0].split(",", 2)[0];
        console.log([ address, num ].join(", "));
        var place = $scope.findPlace([ address, num ].join(", "));
        place.then(function(obj) {
            $scope.confirm(obj[0], true);
        });
        store.set(pos, _.extend(store.get(pos), {
            street_number: num
        }));
    };
} ]);

"use strict";

txsimples.controller("ConfirmCtrl", [ "$scope", "$location", "$timeout", "$ionicPopup", "$q", "Loading", "Ride", function($scope, $location, $timeout, $ionicPopup, $q, Loading, Ride) {
    $scope.price = "0.00";
    $scope.flag = "1";
    $scope.source = "";
    $scope.destination = "";
    $scope.addressObject = function(place) {
        var arr = store.get(place).formatted_address;
        arr = arr.split("-", 1)[0];
        arr = arr.split(",", 2);
        return {
            address: arr[0],
            number: arr[1]
        };
    };
    $scope.render = function() {
        $scope.source = $scope.addressObject("source");
        $scope.destination = $scope.addressObject("destination");
        $scope.price = store.get("estimate").normal_price;
    };
    $scope.requireCab = function() {
        Loading.show();
        var pointsPromise = Ride.submitPlaces([ store.get("source"), store.get("destination") ]);
        pointsPromise.then(function(points) {
            var ride = Ride.requireCab(points);
            ride.then(function(r) {
                Loading.hide();
                $location.url("acompanhar");
            }, function() {
                Loading.hide();
                $ionicPopup.alert({
                    title: "Erro ao pedir corrida",
                    content: "Se o erro persistir, favor nos contactar"
                });
            });
        }, function() {
            Loading.hide();
            $ionicPopup.alert({
                title: "Erro ao pedir corrida",
                content: "Se o erro persistir, favor nos contactar"
            });
        });
    };
    $scope.changeFlag = function() {
        console.log("bandeira");
        $ionicPopup.show({
            title: "Alterar Bandeira.",
            content: "Em qual bandeira deseja exibir a estimativa?",
            buttons: [ {
                text: "Bandeira 1",
                type: "button-peter-rider",
                onTap: function(e) {
                    return 1;
                }
            }, {
                text: "Bandeira 2",
                type: "button-peter-rider",
                onTap: function(e) {
                    return 2;
                }
            } ]
        }).then(function(flag) {
            $scope.flag = flag;
            if (flag == 1) {
                $scope.price = store.get("estimate").normal_price;
            } else {
                $scope.price = store.get("estimate").special_price;
            }
        });
    };
} ]);

txsimples.controller("FollowCtrl", [ "$scope", "$location", "$timeout", "$ionicPopup", "Loading", "Map", "Ride", "RideResource", function($scope, $location, $timeout, $ionicPopup, Loading, Map, Ride, RideResource) {
    var notifications = {};
    $scope.renderStatus = function(ride) {
        var result = "";
        $scope.statusFind = false;
        $scope.statusConfirmed = false;
        $scope.statusRefind = false;
        $scope.statusProgress = false;
        $scope.statusCancelled = false;
        if (ride.indexOf("pendent") !== -1) {
            result = "find";
            $scope.statusFind = true;
        } else if (ride.indexOf("confirmed") !== -1) {
            result = "found";
            $scope.statusConfirmed = true;
        } else if (ride.indexOf("cancelled") !== -1) {
            result = "cancel";
            $scope.statusCancelled = true;
        } else if (ride.indexOf("finished") !== -1) {
            result = "progress";
            $scope.statusProgress = true;
        }
        $(".follow-page").data("message", result);
        return result;
    };
    $scope.checkRide = function() {
        var ride = store.get("ride");
        RideResource.checkRide({
            id: ride.id,
            identifier: ride.identifier
        }, function(ride) {
            Map.addMarker([ ride.response.source.latitude, ride.response.source.longitude, "./img/person.png" ], true);
            if (ride.response.ride.destination) {
                Map.addMarker([ ride.response.ride.destination.lat, ride.response.ride.destination.lng ]);
            }
            if ($scope.statusConfirmed === true) {
                Map.addMarker([ ride.response.cab.lat, ride.response.cab.lng, "./img/car.png" ]);
            }
            var status = ride.response.ride.state;
            $scope.renderStatus(status);
            $scope.ride = ride.response.ride;
            $scope.cab = ride.response.cab;
            if (ride.response.ride.can_finish) {
                if (!notifications[ride.response.ride.id]) {
                    window.plugin.notification.local.schedule({
                        id: ride.response.ride.id,
                        title: "Taxista próximo",
                        text: "Seu taxistas está a menos de 500 metros.",
                        data: {
                            ride_id: ride.response.ride.id
                        }
                    });
                    notifications[ride.response.ride.id] = true;
                }
            }
            if ($scope.ride.state == "cancelled") {
                $ionicPopup.alert({
                    title: "Corrida cancelada",
                    content: "Sua corrida foi cancelada pela central."
                }).then(function() {
                    $location.url("/pedir");
                    store.set("ride", null);
                });
            } else if (store.get("waiting") === true) {
                setTimeout(function() {
                    $scope.checkRide();
                }, app.timeStatusRequest);
            } else {
                $location.url("/pedir");
                store.set("ride", null);
            }
        });
    };
    $scope.rideCancel = function() {
        $ionicPopup.show({
            title: "Confirmação.",
            content: "Tem certeza que deseja cancelar o pedido?",
            buttons: [ {
                text: "Sim",
                type: "button-alizarin",
                onTap: function() {
                    return store.get("ride").id;
                }
            }, {
                text: "Não",
                type: "button-gray",
                onTap: function() {
                    return true;
                }
            } ]
        }).then(function(ride) {
            if (_.isNumber(ride)) {
                Loading.show();
                Ride.cancel(ride).then(function() {
                    Loading.hide();
                    $location.url("/pedir");
                    store.set("ride", null);
                }).catch(function(err) {
                    Loading.hide();
                    $ionicPopup.alert({
                        title: "Erro ao cancelar a corrida.",
                        content: "se o erro persistir, favor nos contactar."
                    });
                });
            }
        });
    };
} ]);

txsimples.controller("IntroCtrl", [ "$scope", "$location", "$timeout", "$q", "$ionicSideMenuDelegate", "Loading", "Facebook", "UserService", "$state", "Protected", function($scope, $location, $timeout, $q, $ionicSideMenuDelegate, Loading, Facebook, UserService, $state, Protected) {
    if (Protected.auth()) {
        $state.go("pedir");
    }
    $ionicSideMenuDelegate.canDragContent(false);
    $timeout(function() {
        $(".splash-screen").hide();
    }, 300);
    $scope.next = function() {
        $timeout(function() {
            $scope.$broadcast("slideBox.nextSlide");
        }, 200);
    };
    $scope.login = function() {
        $(".splash-screen").show();
        Loading.show();
        Facebook.login().then(function(data) {
            UserService.me({
                access_token: data.authResponse.accessToken
            }).$promise.then(function(userData) {
                store.set("user", _.extend(userData, {
                    registered: true
                }));
                $state.go("pedir");
            }).catch(function() {
                $state.go("registro");
            }).finally(function() {
                Loading.hide();
            });
        }).catch(function() {
            Loading.hide();
        });
    };
} ]);

txsimples.controller("AppCtrl", [ "$scope", "$state", "$location", "$timeout", "$ionicSideMenuDelegate", "Protected", function($scope, $state, $location, $timeout, $ionicSideMenuDelegate, Protected) {
    $scope.initialize = function() {
        if ($scope.registered()) {
            $(".splash-screen").hide();
        }
    };
    $scope.toggleMenu = function() {
        $ionicSideMenuDelegate.toggleLeft();
        store.set("toggleMenu", !(store.get("toggleMenu") || false));
    };
    $scope.registered = function() {
        if (Protected.auth() === false) {}
        return Protected.auth();
    };
} ]);

txsimples.controller("MapCtrl", [ "$scope", "$timeout", "$q", "$injector", "$ionicModal", "Map", function($scope, $timeout, $q, $injector, $ionicModal, Map) {
    var loaded = false;
    $ionicModal.fromTemplateUrl("modals.map", function(modal) {
        $scope.modal = modal;
    }, {
        scope: $scope,
        animation: "slide-in-up"
    });
    $scope.initialize = function() {
        Map.initialize();
        Map.clone();
        return true;
    };
    $scope.openModal = function() {
        $scope.modal.show();
        $timeout(function() {
            loaded = loaded == false ? $scope.initialize() : Map.clone();
        }, 200);
    };
    $scope.closeModal = function() {
        $scope.modal.hide();
    };
} ]);

"use strict";

txsimples.controller("RequireCtrl", [ "$scope", "$location", "$timeout", "$q", "$ionicPopup", "Map", "Loading", "Ride", function($scope, $location, $timeout, $q, $ionicPopup, Map, Loading, Ride) {
    if (store.get("ride")) {
        $location.url("acompanhar");
    }
    $scope.togglePlace = function(toggle) {
        if (toggle === true) {
            store.set("togglePlace", "source");
            $scope.showDestination = false;
            $scope.showSource = true;
            if (store.get("source") !== undefined) {
                $scope.render("source");
                $scope.map(store.get("source"));
            }
        } else {
            store.set("togglePlace", "destination");
            $scope.showSource = false;
            $scope.showDestination = true;
            if (store.get("destination") !== undefined) {
                $scope.render("destination");
                $scope.map(store.get("destination"));
            }
        }
    };
    $scope.map = function(place) {
        if (place !== undefined) {
            var location = _.toArray(place.geometry.location);
            Map.addMarker(location, true);
            Map.centerOn(location[0], location[1]);
        } else {
            Map.clearMarkers();
        }
    };
    $scope.render = function(pos) {
        var place = store.get(pos), arr = place.formatted_address.split("-", 1)[0].split(",", 2), address = arr[0], num = parseInt(arr[1]);
        if (!_.isNaN(num)) {
            store.set(pos, _.extend(store.get(pos), {
                street_number: num
            }));
            $timeout(function() {
                $(".enter-" + pos + " .num").val(num);
            }, 500);
        }
        $timeout(function() {
            $(".enter-" + pos + " .address").val(address);
        }, 500);
    };
    $scope.requireEstimate = function() {
        Loading.show();
        if (store.get("source") !== undefined) {
            if (store.get("destination") !== undefined) {
                var source = store.get("source"), destination = store.get("destination");
                if (source.street_number !== undefined && destination.street_number !== undefined) {
                    var price = Ride.estimate();
                    price.then(function() {
                        Loading.hide();
                        $location.url("confirmar");
                    }, function() {
                        Loading.hide();
                        $ionicPopup.alert({
                            title: "Erro ao consultar preço da corrida",
                            content: "Se o erro persistir, favor nos contactar"
                        });
                    });
                } else {
                    Loading.hide();
                    $ionicPopup.show({
                        title: "Pedir Corrida",
                        content: '<i class="ion-4x ion-information-circled"></i> É necessário informar o número do endereço em origem e destino.',
                        buttons: [ {
                            text: "OK",
                            type: "button-peter-rider",
                            onTap: function() {
                                return true;
                            }
                        } ]
                    });
                }
            } else {
                Loading.hide();
                $ionicPopup.show({
                    title: "Antes de Continuar.",
                    content: '<i class="ion-4x ion-help-circled"></i> Sem um endereço de destino preenchido não será possível estimar nem valor nem tempo que levará a corrida. O que deseja fazer?',
                    buttons: [ {
                        text: "Pedir corrida",
                        type: "button-gray",
                        onTap: function() {
                            return "skip";
                        }
                    }, {
                        text: "Preencher",
                        type: "button-peter-rider",
                        onTap: function() {
                            $scope.togglePlace("destination");
                            return true;
                        }
                    } ]
                }).then(function(r) {
                    Loading.show();
                    if (r === "skip") {
                        var source = store.get("source");
                        if (source.street_number !== undefined) {
                            var points = Ride.submitPlaces([ source ]);
                            points.then(function(arr) {
                                var ride = Ride.requireCab(arr);
                                ride.then(function() {
                                    Loading.hide();
                                    $location.url("acompanhar");
                                }, function() {
                                    Loading.hide();
                                    $ionicPopup.alert({
                                        title: "Erro ao pedir corrida",
                                        content: "Se o erro persistir, favor nos contactar"
                                    });
                                });
                            }, function(err) {
                                Loading.hide();
                                $ionicPopup.alert({
                                    title: "Erro ao pedir corrida",
                                    content: "Se o erro persistir, favor nos contactar"
                                });
                            });
                        } else {
                            Loading.hide();
                            $ionicPopup.show({
                                title: "Pedir Corrida",
                                content: '<i class="ion-4x ion-information-circled"></i> É necessário informar o número do endereço em origem.',
                                buttons: [ {
                                    text: "OK",
                                    type: "button-peter-rider",
                                    onTap: function() {
                                        return true;
                                    }
                                } ]
                            });
                        }
                    } else {
                        Loading.hide();
                    }
                });
            }
        } else {
            Loading.hide();
            $ionicPopup.show({
                title: "Pedir Corrida",
                content: '<i class="ion-4x ion-information-circled"></i> Você precisa entrar com um endereço de origem e de destino.',
                buttons: [ {
                    text: "OK",
                    type: "button-peter-rider",
                    onTap: function() {
                        return true;
                    }
                } ]
            });
        }
    };
} ]);

txsimples.controller("UserCtrl", [ "$rootScope", "$scope", "$location", "$q", "$timeout", "$ionicSideMenuDelegate", "$ionicPopup", "UserService", "Loading", "Facebook", "Protected", "Translate", function($rootScope, $scope, $location, $q, $timeout, $ionicSideMenuDelegate, $ionicPopup, UserService, Loading, Facebook, Protected, Translate) {
    $scope.companies = store.get("companies");
    $scope.user = Protected.getUser();
    Facebook.getAccessToken().then(function(token) {
        UserService.me({
            access_token: token
        }).$promise.then(function(userData) {
            store.set("user", _.extend(userData, {
                registered: true
            }));
            $scope.user = Protected.getUser();
        }, function() {
            Facebook.api("me/?fields=id,name,email").then(function(data) {
                $scope.user.name = data.name;
                $scope.user.email = data.email;
            });
        });
    });
    $scope.create = function() {
        if ($scope.user.phone !== "") {
            Loading.show();
            Facebook.getAccessToken().then(function(token) {
                $scope.user.access_token = token;
                UserService.join($scope.user, function(user) {
                    Loading.hide();
                    store.set("user", _.extend(user, {
                        registered: true
                    }));
                    window.location.hash = "#/pedir";
                }, function(err) {
                    Loading.hide();
                    var msgs = [];
                    for (var i in err.data.error) {
                        msgs.push(i + ": " + Translate.msg(err.data.error[i]));
                    }
                    $ionicPopup.alert({
                        title: "Erro ao criar conta",
                        content: msgs.join("<br />")
                    });
                });
            });
        } else {
            alert("Informe um telefone válido!");
        }
    };
    $scope.getCompanies = function() {
        UserService.getCompanies({}, function(companies) {
            console.log(companies);
            store.set("companies", companies);
        });
    };
    $scope.updateProfile = function() {
        console.log($scope.user);
        Loading.show();
        Facebook.getAccessToken().then(function(token) {
            $scope.user.access_token = token;
            UserService.edit({
                id: $scope.user.id
            }, $scope.user, function(user) {
                store.set("user", _.extend(user, {
                    registered: true
                }));
                window.location.hash = "#/pedir";
                Loading.hide();
            }, function(err) {
                Loading.hide();
                alert(err);
            });
        });
    };
    $scope.logout = function() {
        Loading.show();
        store.set("user", Protected.user);
        store.set("facebook_accessToken", null);
        $timeout(function() {
            Loading.hide();
            window.location.hash = "#/intro";
        }, 500);
    };
} ]);