var source = new google.maps.Geocoder();

txsimples.controller('AddressCtrl', [
    '$scope', 
    '$q',
    '$timeout',
    '$ionicModal',
    '$ionicPopup',
    'Map',
    'Loading',
    'Protected',

    function($scope, $q, $timeout, $ionicModal, $ionicPopup, Map, Loading, Protected) {
        $scope.address = undefined;
        $scope.addresses = [];
        
        $scope.user = Protected.getUser();
        $scope.points = $scope.user.favorites;

        $ionicModal.fromTemplateUrl('modals.address', function(modal) {
            $scope.modal = modal;
        }, {
            scope: $scope,
            animation: 'active'
        });

        $scope.selectPoint = function() {
            var place = {  
              "address_components":[  
                {  
                  "long_name": $scope.currentPoint.number,
                  "short_name": $scope.currentPoint.number,
                  "types":[  
                    "street_number"
                  ]
                },
                {  
                  "long_name": $scope.currentPoint.street,
                  "short_name": $scope.currentPoint.street,
                  "types":[  
                    "route"
                  ]
                },
                {  
                  "long_name": $scope.currentPoint.neighborhood,
                  "short_name": $scope.currentPoint.neighborhood,
                  "types":[  
                    "neighborhood",
                    "political"
                  ]
                },
                {  
                  "long_name":$scope.currentPoint.city,
                  "short_name":$scope.currentPoint.city,
                  "types":[  
                    "locality",
                    "political"
                  ]
                },
                {  
                  "long_name":$scope.currentPoint.city,
                  "short_name":$scope.currentPoint.city,
                  "types":[  
                    "administrative_area_level_2",
                    "political"
                  ]
                },
                {  
                  "long_name":$scope.currentPoint.state,
                  "short_name":$scope.currentPoint.state,
                  "types":[  
                    "administrative_area_level_1",
                    "political"
                  ]
                },
                {  
                  "long_name":"Brasil",
                  "short_name":"BR",
                  "types":[  
                    "country",
                    "political"
                  ]
                },
              ],
              "formatted_address":$scope.currentPoint.street+", "+$scope.currentPoint.number+" - "+$scope.currentPoint.neighborhood+", "+$scope.currentPoint.city+"- "+$scope.currentPoint.state+", Brasil",
              "geometry":{  
                "location":{  
                  "k":$scope.currentPoint.latitude,
                  "D":$scope.currentPoint.longitude
                },
                "location_type":"RANGE_INTERPOLATED",
              },
              "types":[  
                "street_address"
              ],
              "street_number":$scope.currentPoint.number
            }
            $scope.confirm(place, true);
            console.log($scope.currentPoint);
            $scope.currentPoint = null;
        }

        $scope.openModal = function() {
            console.log($scope.modal.el);
            $scope.modal.show();
        };

        $scope.closeModal = function() {
            console.log($scope.modal.el);
            $scope.modal.hide();
        };

        // Capta a posição do usuário por geolocation.

        $scope.currentPosition = function() {
            Loading.show();

            ( Map.currentPosition() ).then(function(position) {
                ( Map.geocode(position) ).then(function(place) {
                    $scope.confirm(place);
                    Loading.hide();
                }, function(){
                  Loading.hide();
                  $ionicPopup.alert({
                      title: 'Endereço não encontrado.',
                      content: 'Não pudemos encontrar um endereço para sua localição.'
                  });  
                });
            }, function() {
                Loading.hide();
                $ionicPopup.show({
                    title: 'Serviço de Localização desativado.',
                    content: 'Essa função requer que você use o serviço de localização do seu dispositivo.',
                    buttons: [
                        { text: 'OK', type: 'button-peter-rider', onTap: function(e) { return true; } }
                    ]
                });
            });
        };

        // Pesquisa por um endereço qualquer.

        $scope.findPlace = function(address) {

            var r = $q.defer();
            
            source.geocode({
                'address': address + ", Fortaleza, Ceara, Brasil",
                'language': 'PT-BR'
            },function(results, status) {
                if (status == google.maps.GeocoderStatus.OK) {
                    $scope.addresses = [];
                    _.each(results, function(result) {
                        $scope.addresses.push(result);
                    });
                    r.resolve(results);
                }
            });

            return r.promise;
        };

        // Renderiza um marcador no mapa.

        $scope.renderInMap = function(place) {
            var location = _.toArray(place.geometry.location);

            Map.addMarker(location, true);
            Map.centerOn(location[0], location[1]);
        };

        // Confirma uma escolha de um ponto para armazenar e renderizar marcador.

        $scope.confirm = function(place, renderIt) {
            var arr     = ( place.formatted_address.split('-', 1)[0] ).split(',', 2),
                pos     = store.get('togglePlace'),
                address = arr[0],
                num     = parseInt(arr[1]);

            $scope.closeModal();

            // Persiste em localStorage de acordo com a 
            // opção corrente de local (origem ou destino).

            store.set(pos, place);

            if(renderIt) {
                $scope.renderInMap(place);
            }

            // Confere se o retorno é um número.

            if(!_.isNaN(num)) {

                // Persiste o número em localStorage.
                
                store.set(pos, _.extend(store.get(pos), {"street_number": num}));
                
                $timeout(function() {
                    $('.enter-' + pos + ' .num').val(num);
                }, 500);
            }
            
            $timeout(function() {
                $('.enter-' + pos + ' .address').val(address);
            }, 500);
        };

        // Atualiza com uma nova busca e volta a persistir.
        $scope.$watch('num', function(num){
          if (num) {
            $scope.update(num);
          }
        })
        $scope.update = function(num) {

            // Recebe o numero como parametro
            // ou permanece com o mesmo do localStorage.

            var pos     = store.get('togglePlace'),
                address = ( ( store.get(pos) ).formatted_address.split('-', 1)[0] ).split(',', 2)[0];

            console.log([address, num].join(", "));

            var place = $scope.findPlace( [address, num].join(", ") );
            
            place.then(function(obj) {
                $scope.confirm(obj[0], true);
            })

            store.set(pos, _.extend(store.get(pos), {street_number: num}));
        }

    }
    
]);
