'use strict';

txsimples.controller('RequireCtrl', [
    '$scope',
    '$location',
    '$timeout',
    '$q',
    '$ionicPopup',
    'Map',
    'Loading',
    'Ride',

    function($scope, $location, $timeout, $q, $ionicPopup, Map, Loading, Ride) {

        if (store.get('ride')) {
            $location.url('acompanhar');
        }
        // Configura a entrada corrente por posicao atual,
        
        $scope.togglePlace = function(toggle) {

            if(toggle===true) {
                store.set('togglePlace', 'source');

                $scope.showDestination = false;
                $scope.showSource = true;
                
                if(store.get('source') !== undefined) {
                    $scope.render('source');
                    $scope.map( store.get('source') );
                }
            
            } else {
                store.set('togglePlace', 'destination');

                $scope.showSource = false;
                $scope.showDestination = true;

                if(store.get('destination') !== undefined) {
                    $scope.render('destination');
                    $scope.map( store.get('destination') );
                }
            }
        };

        // Renderiza um ponto qualquer no mapa.
        
        $scope.map = function(place) {
            
            if(place !== undefined) {
                var location = _.toArray(place.geometry.location);

                Map.addMarker(location, true);
                Map.centerOn(location[0], location[1]);
            } else {
                Map.clearMarkers();
            }
        };

        // Renderiza um ponto a partir da escolha de
        // um ponto ou por geoprocessamento.

        $scope.render = function(pos) {

            var place   = store.get(pos),
                arr     = ( place.formatted_address.split('-', 1)[0] ).split(',', 2),
                address = arr[0],
                num     = parseInt(arr[1]);

            if(!_.isNaN(num)) {

                // Persiste o número em localStorage.
                
                store.set(pos, _.extend(store.get(pos), {'street_number': num}));
                
                $timeout(function() {
                    $('.enter-' + pos + ' .num').val(num);
                }, 500);
            }
            
            $timeout(function() {
                $('.enter-' + pos + ' .address').val(address);
            }, 500);
        };

        // Verificação do preenchimento

        $scope.requireEstimate = function() {

            Loading.show();

            // Se houve entrada ao menos do endereço de destino.

            if(store.get('source') !== undefined) {

                // Se não houve entrada de destino então deve escolher 
                // se irá requisitar corrida sem estimativa.

                if(store.get('destination') !== undefined) {

                    var source      = store.get('source'),
                        destination = store.get('destination');

                    // Verifica se há a entrada dos numerais do pontos informados.

                    if( source.street_number !== undefined && destination.street_number !== undefined ) {

                        var price = Ride.estimate();

                        price.then(function() {

                            Loading.hide();

                            $location.url('confirmar');
                        }, function(){
                            Loading.hide();
                            $ionicPopup.alert({
                                title: 'Erro ao consultar preço da corrida',
                                content: 'Se o erro persistir, favor nos contactar'
                            })
                        });

                    } else {

                        Loading.hide();
                        
                        $ionicPopup.show({
                            title: 'Pedir Corrida',
                            content: '<i class="ion-4x ion-information-circled"></i> É necessário informar o número do endereço em origem e destino.',
                            buttons: [
                                { text: 'OK', type: 'button-peter-rider', onTap: function() { return true; } }
                            ]
                        });
                    }

                } else {

                    Loading.hide();

                    $ionicPopup.show({
                        title: 'Antes de Continuar.',
                        content: '<i class="ion-4x ion-help-circled"></i> Sem um endereço de destino preenchido não será possível estimar nem valor nem tempo que levará a corrida. O que deseja fazer?',
                        buttons: [
                            { text: 'Pedir corrida', type: 'button-gray', onTap: function() { return 'skip'; } },
                            { text: 'Preencher', type: 'button-peter-rider', onTap: function() { $scope.togglePlace('destination'); return true; } }
                        ]
                    }).then(function(r) {

                        Loading.show();

                        // Continua a preenche para completar estimativa ou segue com o pedido.

                        if(r === 'skip') {

                            var source = store.get('source');

                            // Verifica se há a entrada do número do endereço de origem.

                            if( source.street_number !== undefined ) {

                                var points = Ride.submitPlaces([source]);

                                points.then(function(arr) {
                                    var ride = Ride.requireCab(arr);
                                    ride.then(function() {
                                        Loading.hide();
                                        $location.url('acompanhar');
                                    }, function(){
                                        Loading.hide();
                                        $ionicPopup.alert({
                                            title: 'Erro ao pedir corrida',
                                            content: 'Se o erro persistir, favor nos contactar'
                                        })
                                    });
                                }, function(err){
                                    Loading.hide();
                                    $ionicPopup.alert({
                                        title: 'Erro ao pedir corrida',
                                        content: 'Se o erro persistir, favor nos contactar'
                                    })
                                });
                                
                            } else {

                                Loading.hide();
                        
                                $ionicPopup.show({
                                    title: 'Pedir Corrida',
                                    content: '<i class="ion-4x ion-information-circled"></i> É necessário informar o número do endereço em origem.',
                                    buttons: [
                                        { text: 'OK', type: 'button-peter-rider', onTap: function() { return true; } }
                                    ]
                                });
                            }

                        } else {

                            Loading.hide();

                        }
                    });

                }

            } else {

                Loading.hide();
                
                $ionicPopup.show({
                    title: 'Pedir Corrida',
                    content: '<i class="ion-4x ion-information-circled"></i> Você precisa entrar com um endereço de origem e de destino.',
                    buttons: [
                        { text: 'OK', type: 'button-peter-rider', onTap: function() { return true; } }
                    ]
                });

            }
        };

    }
    
]);
