txsimples.service('Map', function ($q, $timeout, Ride) {
    var loc      = [0, 0],
        markers  = [],
        geocoder = undefined,
        scope    = this;

    this.initialize = function($q) {
        geocoder = new google.maps.Geocoder();

        var mapOptions = {
            center: new google.maps.LatLng(-3.7630658, -38.5211885),
            zoom: 13,
            disableDefaultUI: true,
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };

        scope.map = [];

        _.filter( document.getElementsByClassName('map'), function(elem) {
            scope.map.push( new google.maps.Map(elem, mapOptions) );
        });
    };

    // Atualiza o mapa.

    this.update = function() {
        _.each(scope.map, function(map) {
            google.maps.event.trigger(map, 'resize');
        });
    };

    // Referencia a lista de markers em um mapa.

    this.setMarkers = function(map) {
     
        for (var i = 0; i < markers.length; i++) {
            markers[i].setMap(map);
        }
    };

    // Limpa todos os marcadores da lista.

    this.clearMarkers = function() {
        scope.setMarkers(null);
    };

    // Cria um novo marcador ao mapa, removendo (ou não)
    // os marcadores anteriores.  

    this.addMarker = function(loc, clear) {
        if( clear ) {
            scope.clearMarkers();
        }
        var marker = {
            position: new google.maps.LatLng(loc[0], loc[1]),
            draggable: false,
            //animation: google.maps.Animation.DROP,
            //icon: 'http://galiciamotorsltdkensington.co.uk/communities/2/000/001/036/102//images/1920192.png',
            //shadow: shadow
        }
        if (loc[2]) {
            marker.icon = loc[2];
        }

        // Incluir o ponto em todos os mapas.
        _.each(scope.map, function(map) {
            marker.map = map;
            markers.push(new google.maps.Marker(marker));
        });
        
        store.set('currentMarker', loc);
    };

    // Obtém a posição do usuário fornecido pelo dispositivo
    // ou pedindo autorização ao navegador.

    this.getCurrentPosition = function() {
        var position = $q.defer();

        navigator.geolocation.getCurrentPosition(function(pos) {
            position.resolve(pos);
        }, function(err) {
            position.reject(err); 
        }, {
            enableHighAccuracy: true
        });

        $timeout(function(){
            position.reject({timeout: true});
        }, 5000);

        return position.promise;
    };

    // Centraliza o mapa para a posição entrada ou atual.

    this.centerOn = function(lat, lng) {
        if( lat && lng ) {
            var loc = new google.maps.LatLng(lat, lng);
            _.each(scope.map, function(map) {
                map.setCenter(loc);
                map.panTo(loc);
            });
        } else {
            scope.getCurrentPosition(function(lat, lng) {
                var loc = new google.maps.LatLng(lat, lng);
                _.each(scope.map, function(map) {
                    map.setCenter(loc);
                    map.panTo(loc);
                });
            });
        }
    };

    // Intermedia a solicitação da posição atual.

    this.currentPosition = function() {
        var position = scope.getCurrentPosition(),
            deferred = $q.defer();

        position.then(function(pos) {
            var pos = { "lat": pos.coords.latitude, "lng": pos.coords.longitude };

            scope.addMarker([pos.lat, pos.lng], true);
            scope.centerOn(pos.lat, pos.lng);

            // Retorna latitude e longitude quando 
            // este estiver pronto.
            
            deferred.resolve( pos );
        }, function() {
            deferred.reject();
        });

        scope.update();

        return deferred.promise;
    };

    // Encontra um endereco correspondente proximo a um dado ponto.

    this.geocode = function(pos) {
        var position  = new google.maps.LatLng(pos.lat, pos.lng),
            address   = $q.defer();

        geocoder.geocode({'latLng': position}, function(results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                var resolved = false;
                angular.forEach(results, function(result){
                    if (!resolved) {
                        var locate = Ride.placeObject(result);
                        if (locate.city && locate.street && locate.neighborhood) {
                            address.resolve(result);
                            resolved = true;
                        }
                    }
                })
                if (!resolved) {
                    address.reject();
                }
            } else {
                alert('Geocoder failed due to: ' + status);
            }
        });

        return address.promise;
    };

    // Repassa o marcador corrente a todos os mapas.

    this.clone = function() {
        var loc = store.get('currentMarker');

        if(loc !== undefined) {
            this.addMarker(loc, true);
            this.centerOn(loc[0], loc[1]);
        }
    };
});
