'use strict';

txsimples.service('Ride', function ($q, $resource, RideResource, Facebook) {

  var self = this;

  var directionService = new google.maps.DirectionsService();

  // Retorna objeto com polilinhas e informações de 
  // distancia do percurso.

  this.route = function(source, destination) {

    var route = $q.defer();

    directionService.route({
    
      origin:      new google.maps.LatLng(source[0], source[1]),
      destination: new google.maps.LatLng(destination[0], destination[1]),
      travelMode:  google.maps.DirectionsTravelMode.DRIVING
    
    }, function(res, status) {
      
      // Verifica confirmação do status da resposta.

      if (status == google.maps.DirectionsStatus.OK) {
        var r = {};

        // Pega apenas a primeira opção de rota.
       
        r.distance = ( res.routes[0].legs[0] ).distance.value;
        r.distance = ( r.distance ).toFixed(2);

        r.directions = res;

        route.resolve(r);
      } else {
        route.reject();
      }
    });

    return route.promise;
  };

  // Retorna estimativa de preço recebendo a cidade 
  // e a distancia a ser percorrida.

  this.price = function(city, distance) {
    
    var price = $q.defer();

    RideResource.estimatePrice({distance: distance, city: city}, function(r) {
      price.resolve(r);
    },function(err){
      price.reject(err);
    });

    return price.promise;
  };

  // Formata o objeto de um local do Google Maps 
  // para responder ao servidor.
  
  this.placeObject = function(point) {

    var address  = point.address_components,
      location = _.toArray(point.geometry.location),
      result   = {};

    result.latitude  = location[0];
    result.longitude = location[1];

    for (var i = 0; i < address.length; i++) {

      if (address[i].types.indexOf("route") != -1) {
        
        result.street = address[i].long_name;

      } else if (address[i].types.indexOf("street_number") != -1) {
        
        result.number = address[i].long_name;

      } else if (address[i].types.indexOf("sublocality") != -1) {
        
        result.neighborhood = address[i].long_name;

      } else if (address[i].types.indexOf("neighborhood") != -1) {
        
        result.neighborhood = address[i].long_name;

      } else if (address[i].types.indexOf("locality") != -1) {
        
        result.city = address[i].long_name;

      } else if (address[i].types.indexOf("administrative_area_level_1") != -1) {
        
        result.state = address[i].long_name;
      
      }
    };

    return result;
  };

  // Submete ao servidor origem e destino criando 
  // uma requisição de corrida.

  this.estimate = function() {

    var estimate = $q.defer();

    var source = self.placeObject( store.get('source') );

    var route = self.route(
      _.toArray(store.get('source').geometry.location), 
      _.toArray(store.get('destination').geometry.location) 
    );

    route.then(function(route) {
      var price = self.price( source.city, route.distance );

      price.then(function(r) {                
        store.set('estimate', r.response);
        estimate.resolve(r);                
      }, function(err){
        estimate.reject(err);
      });
    });

    return estimate.promise;
  };

  // Submete ao servidor da aplicação os locais usados
  // pelo usuário na corrida.

  this.submitPlaces = function(arrs) {

    var points = [];

    angular.forEach(arrs, function(place) {
      var deferred = $q.defer();
    
      Facebook.getAccessToken()
      .then(function(token){
        RideResource.createPoint({
          access_token: token,
          point: self.placeObject( place )

        }, function(r) {  
          deferred.resolve(r);
        }, function(err){
          deferred.reject(err);
        });
      });
      points.push( deferred.promise );
    });

    return $q.all(points);
  };

  // Submete uma corrida ao servidor que passará a
  // aguardar a aceitação por um taxista.

  this.requireCab = function(points) {

    var result      = $q.defer(),
      destination = undefined;

    if(points.length > 1) {
      destination = points[1].id;
    }

    var ride     = $q.defer(),
      rideData = {
        source_id:      points[0].id,
        destination_id: destination,
        description:    "[description]"
    };

    Facebook.getAccessToken()
    .then(function(token){
      rideData.access_token = token;
      RideResource.callRide(rideData, function(res) {
        store.set('ride', res);
        store.set('waiting', true);
        result.resolve(res);
      }, function(err){
        result.reject(err);
      });
    });

    return result.promise;
  };

  // Solicita cancelamento do pedido de corrida em
  // qualquer etapa no processo. 

  this.cancel = function(ride) {

    var deferred = $q.defer();
    Facebook.getAccessToken()
    .then(function(token){
      RideResource.cancel({
        'id': ride,
        access_token: token
      }, function(r) {
        store.set("waiting", false);
        deferred.resolve(r);
      }, function(err) {
        deferred.reject(err);
      });
    });

    return deferred.promise;
  };

});
